import sys, subprocess as sps, os

def run(options, start_urls):
    chrome_path = get_instance_path()

    if chrome_path is not None:
        if options['mode'] == 'chrome-app':
            for url in start_urls:
                sps.Popen([chrome_path, '--app=%s' % url] +
                           options['chromeFlags'],
                           stdout=sps.PIPE, stderr=sps.PIPE, stdin=sps.PIPE)
        else:
            args = options['chromeFlags'] + start_urls
            sps.Popen([chrome_path, '--new-window'] + args,
                       stdout=sps.PIPE, stderr=sps.PIPE, stdin=sps.PIPE)
    else:
        raise EnvironmentError(
            "Can't find Chrome or Chromium installation")


def get_instance_path():
    if sys.platform in ['win32', 'win64']:
        return find_chrome_win()
    elif sys.platform == 'darwin':
        return find_chrome_mac()
    elif sys.platform.startswith('linux'):
        return find_chrome_linux()
    else:
        return None


def find_chrome_mac():
    default_dir = r'/Applications/Google Chrome.app/Contents/MacOS/Google Chrome'
    if os.path.exists(default_dir):
        return default_dir
    # use mdfind ci to locate Chrome in alternate locations and return the first one
    name = 'Google Chrome.app'
    alternate_dirs = [x for x in sps.check_output(["mdfind", name]).decode().split('\n') if x.endswith(name)] 
    if len(alternate_dirs):
        return alternate_dirs[0] + '/Contents/MacOS/Google Chrome'
    return None


def find_chrome_linux():
    import whichcraft as wch
    chrome_names = ['chromium-browser',
                    'chromium',
                    'google-chrome',
                    'google-chrome-stable']

    for name in chrome_names:
        chrome = wch.which(name)
        if chrome is not None:
            return chrome
    return None


def find_chrome_win():
    import winreg as reg
    reg_path = r'SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\chrome.exe'

    for install_type in reg.HKEY_CURRENT_USER, reg.HKEY_LOCAL_MACHINE:
        try:
            reg_key = reg.OpenKey(install_type, reg_path, 0, reg.KEY_READ)
            chrome_path = reg.QueryValue(reg_key, None)
            reg_key.Close()
        except WindowsError:
            chrome_path = None
        else:
            break

    return chrome_path



import webbrowser as wbr
import eel.chrome as chm
import eel.electron as ele
import subprocess as sps

def _build_url_from_dict(page, options):
    scheme = page.get('scheme', 'http')
    host = page.get('host', 'localhost')
    port = page.get('port', 8000)
    path = page.get('path', '')
    return '%s://%s:%d/%s' % (scheme, host, port, path)


def _build_url_from_string(page, options):
    base_url = 'http://%s:%d/' % (options['host'], options['port'])
    return base_url + page


def _build_urls(start_pages, options):
    urls = []

    for page in start_pages:
        method = _build_url_from_dict if isinstance(
            page, dict) else _build_url_from_string
        url = method(page, options)
        urls.append(url)

    return urls


def open(start_pages, options):
    start_urls = _build_urls(start_pages, options)

    if options['mode'] in ['chrome', 'chrome-app']:
        chm.run(options, start_urls)
    elif options['mode'] in [None, False]:
        pass  # Don't open a browser
    elif options['mode'] == 'electron':
        ele.run(options, start_urls)
    elif options['mode'] == 'custom':
        sps.Popen(options['args'],
                  stdout=sps.PIPE, stderr=sps.PIPE, stdin=sps.PIPE)
    else:
        # Use system default browser
        for url in start_urls:
            wbr.open(url)



from __future__ import print_function
import sys
import pkg_resources as pkg
import PyInstaller.__main__ as pyi
import os

args = sys.argv[1:]
main_script = args.pop(0)
web_folder = args.pop(0)

print("Building executable with main script '%s' and web folder '%s'...\n" %
      (main_script, web_folder))

eel_js_file = pkg.resource_filename('eel', 'eel.js')
js_file_arg = '%s%seel' % (eel_js_file, os.pathsep)
web_folder_arg = '%s%s%s' % (web_folder, os.pathsep, web_folder)

needed_args = ['--hidden-import', 'bottle_websocket',
               '--add-data', js_file_arg, '--add-data', web_folder_arg]
full_args = [main_script] + needed_args + args

print('Running:\npyinstaller', ' '.join(full_args), '\n')

pyi.run(full_args)



import subprocess as sps

def run(options, start_urls):
	# TODO: Find Eletron properly, e.g. on Windows

	electron_path = '/usr/local/bin/electron'
	sps.Popen([electron_path, '.'] + start_urls,
              stdout=sps.PIPE, stderr=sps.PIPE, stdin=sps.PIPE)

